<?php

namespace Drupal\twicpics\Service;

/**
 * The MappingImageStyle service. Does a bunch of mapping style.
 */
class MappingImageStyle {

  private $arrayMappingEffect =
  [
    "crop_crop" => "crop",
    "image_convert" => "convert",
    "image_crop" => "crop",
    "image_desaturate" => "",
    "image_scale" => "contain",
    "image_scale_and_crop" => "cover",
    "image_resize" => "resize",
    "image_rotate" => "rotate"
  ];

  private $arrayMappingFocus = [
    "left-top" => "top-left",
    "center-bottom" => "top",
    "right-top" => "top-right",
    "left-center" => "left",
    "center-center" => "center",
    "right-center" => "right",
    "left-bottom" => "bottom-left",
    "center-bottom" => "bottom",
    "right-bottom" => "bottom-right",
  ];

  private $arrayMappingOutput = [
    "jpeg" => "jpeg",
    "jpg" => "jpeg",
    "png" => "png",
    "webp" => "webp",
  ];

  /**
   * Does something.
   *
   * @return string
   *   Some value.
   */
  public function convert( $effects ) {

    $paramsUrl = "";

    foreach( $effects as $effect ) {

      if ( array_key_exists( $effect->getSummary()['#effect']['id'], $this->arrayMappingEffect ) ) {

        $twicpicsEffect = $this->arrayMappingEffect[$effect->getSummary()['#effect']['id']];
        if ( $twicpicsEffect != "" ) {
          $actualCover = "";
          $actualContain = "";
          $actualContainMax = "";
          $actualCrop = "";
          $actualFocus = "";
          $actualOutput = "";
          $actualResize = "";
          $actualTransform = "";

          $drupalData = array_key_exists( '#data', $effect->getSummary() ) ?
            $effect->getSummary()['#data']:
            [];

          //Focus
          if (
              array_key_exists( 'anchor', $drupalData ) && 
              array_key_exists( $drupalData['anchor'], $this->arrayMappingFocus )
            ) {
            $actualFocus = "/focus=".$this->arrayMappingFocus[ $drupalData[ 'anchor' ] ];
          }
          
          //Scale Effect
          if ( $twicpicsEffect == "contain" ) {
            $actualContain = \Drupal::service( 'compute' )->computeSizeAttribute(
              "contain",
              $drupalData[ 'width' ],
              $drupalData[ 'height' ], 
            );

            if ( !$drupalData['upscale'] ) {
              $actualContainMax = \Drupal::service( 'compute' )->computeSizeAttribute(
                "contain-max",
                $drupalData[ 'width' ],
                $drupalData[ 'height' ], 
              );
            }
          }

          //Convert Effect
          if ( $twicpicsEffect == "convert" ) {
            $drupalMarkup = strtolower( $effect->getSummary()[ '#markup' ] );
            if ( array_key_exists( $drupalMarkup, $this->arrayMappingOutput ) ) {
              $actualOutput = "/output=".$this->arrayMappingOutput[ $drupalMarkup ];
            }
          }

          //Cover Effect
          if ( $twicpicsEffect == "cover" ) {
            $actualCrop = \Drupal::service( 'compute' )->computeSizeAttribute(
              "cover",
              $drupalData[ 'width' ],
              $drupalData[ 'height' ], 
            );
          }

          //Crop Effect
          if ( $twicpicsEffect == "crop" && !array_key_exists( 'crop_type', $drupalData ) ) {
              $actualCrop = \Drupal::service( 'compute' )->computeSizeAttribute(
                "crop",
                $drupalData[ 'width' ],
                $drupalData[ 'height' ], 
              );
          }

          //Resize Effect
          if ($twicpicsEffect == "resize") {
            $actualResize = \Drupal::service( 'compute' )->computeSizeAttribute(
              "resize",
              $drupalData[ 'width' ],
              $drupalData[ 'height' ], 
            );
          }

          //Rotate Effect
          if ( $twicpicsEffect == "rotate" ) {
              $actualTransform = "/turn=".(
                $drupalData['random'] ?
                rand(0, $drupalData['degrees']) :
                $drupalData['degrees']
              );
          }

          $paramsUrl.=
            $actualFocus.
            $actualCover.
            $actualContain.
            $actualContainMax.
            $actualCrop.
            $actualResize.
            $actualTransform.
            $actualOutput;
        }
      }
    }
    return $paramsUrl;
  }
}
